@@ -928,8 +928,12 @@ GDALDataset *EHdrDataset::Open(GDALOpenInfo *poOpenInfo, bool bFileSizeCheck)
 
 {
     // Assume the caller is pointing to the binary (i.e. .bil) file.
-    if (poOpenInfo->nHeaderBytes < 2 || poOpenInfo->fpL == nullptr)
+    if (poOpenInfo->nHeaderBytes < 2 || poOpenInfo->fpL == nullptr ||
+        (!poOpenInfo->IsSingleAllowedDriver("EHdr") &&
+         poOpenInfo->IsExtensionEqualToCI("zarr")))
+    {
         return nullptr;
+    }
 
     // Tear apart the filename to form a .HDR filename.
     const CPLString osPath = CPLGetPathSafe(poOpenInfo->pszFilename);
@@ -947,7 +951,7 @@ GDALDataset *EHdrDataset::Open(GDALOpenInfo *poOpenInfo, bool bFileSizeCheck)
         pszHeaderExt = "sch";
     }
 
-    char **papszSiblingFiles = poOpenInfo->GetSiblingFiles();
+    CSLConstList papszSiblingFiles = poOpenInfo->GetSiblingFiles();
     CPLString osHDRFilename;
     if (papszSiblingFiles)
     {
@@ -1683,27 +1687,39 @@ GDALDataset *EHdrDataset::Create(const char *pszFilename, int nXSize,
         return nullptr;
 
     // Create the hdr filename.
-    char *const pszHdrFilename =
-        CPLStrdup(CPLResetExtensionSafe(pszFilename, "hdr").c_str());
+    const std::string osHdrFilename = CPLResetExtensionSafe(pszFilename, "hdr");
 
     // Open the file.
-    fp = VSIFOpenL(pszHdrFilename, "wt");
+    fp = VSIFOpenL(osHdrFilename.c_str(), "wt");
     if (fp == nullptr)
     {
         CPLError(CE_Failure, CPLE_OpenFailed,
-                 "Attempt to create file `%s' failed.", pszHdrFilename);
-        CPLFree(pszHdrFilename);
+                 "Attempt to create file `%s' failed.", osHdrFilename.c_str());
         return nullptr;
     }
 
     // Decide how many bits the file should have.
-    int nBits = GDALGetDataTypeSize(eType);
-
-    if (CSLFetchNameValue(papszParamList, "NBITS") != nullptr)
-        nBits = atoi(CSLFetchNameValue(papszParamList, "NBITS"));
+    const char *pszNBITS = CSLFetchNameValue(papszParamList, "NBITS");
+    const int nBits =
+        pszNBITS ? atoi(pszNBITS) : GDALGetDataTypeSizeBits(eType);
+    if (nBits <= 0 || nXSize > (static_cast<int64_t>(INT_MAX) * 8 - 7) / nBits)
+    {
+        CPLError(CE_Failure, CPLE_AppDefined,
+                 "Invalid NBITS or too large image width");
+        CPL_IGNORE_RET_VAL(VSIFCloseL(fp));
+        return nullptr;
+    }
 
     const int nRowBytes = (nBits * nXSize + 7) / 8;
 
+    if (nRowBytes > INT_MAX / nBandsIn)
+    {
+        CPLError(CE_Failure, CPLE_AppDefined,
+                 "Too large band count and/or image width");
+        CPL_IGNORE_RET_VAL(VSIFCloseL(fp));
+        return nullptr;
+    }
+
     // Check for signed byte.
     const char *pszPixelType = CSLFetchNameValue(papszParamList, "PIXELTYPE");
     if (pszPixelType == nullptr)
@@ -1731,8 +1747,6 @@ GDALDataset *EHdrDataset::Create(const char *pszFilename, int nXSize,
     if (VSIFCloseL(fp) != 0)
         bOK = false;
 
-    CPLFree(pszHdrFilename);
-
     if (!bOK)
         return nullptr;
 
