@@ -473,8 +473,12 @@ CPLErr PAuxDataset::GetGeoTransform(double *padfGeoTransform)
 GDALDataset *PAuxDataset::Open(GDALOpenInfo *poOpenInfo)
 
 {
-    if (poOpenInfo->nHeaderBytes < 1)
+    if (poOpenInfo->nHeaderBytes < 1 ||
+        (!poOpenInfo->IsSingleAllowedDriver("PAux") &&
+         poOpenInfo->IsExtensionEqualToCI("zarr")))
+    {
         return nullptr;
+    }
 
     /* -------------------------------------------------------------------- */
     /*      If this is an .aux file, fetch out and form the name of the     */
@@ -512,7 +516,7 @@ GDALDataset *PAuxDataset::Open(GDALOpenInfo *poOpenInfo)
     /* -------------------------------------------------------------------- */
     /*      Do we have a .aux file?                                         */
     /* -------------------------------------------------------------------- */
-    char **papszSiblingFiles = poOpenInfo->GetSiblingFiles();
+    CSLConstList papszSiblingFiles = poOpenInfo->GetSiblingFiles();
     if (papszSiblingFiles != nullptr &&
         CSLFindString(papszSiblingFiles, CPLGetFilename(osAuxFilename)) == -1)
     {
