@@ -48,9 +48,9 @@ std::shared_ptr<ZarrArray> ZarrV3Group::OpenZarrArray(const std::string &osName,
         return oIter->second;
 
     const std::string osSubDir =
-        CPLFormFilename(m_osDirectoryName.c_str(), osName.c_str(), nullptr);
+        CPLFormFilenameSafe(m_osDirectoryName.c_str(), osName.c_str(), nullptr);
     const std::string osZarrayFilename =
-        CPLFormFilename(osSubDir.c_str(), "zarr.json", nullptr);
+        CPLFormFilenameSafe(osSubDir.c_str(), "zarr.json", nullptr);
 
     VSIStatBufL sStat;
     if (VSIStatL(osZarrayFilename.c_str(), &sStat) == 0)
@@ -76,7 +76,7 @@ void ZarrV3Group::LoadAttributes() const
     m_bAttributesLoaded = true;
 
     const std::string osFilename =
-        CPLFormFilename(m_osDirectoryName.c_str(), "zarr.json", nullptr);
+        CPLFormFilenameSafe(m_osDirectoryName.c_str(), "zarr.json", nullptr);
 
     VSIStatBufL sStat;
     if (VSIStatL(osFilename.c_str(), &sStat) == 0)
@@ -106,11 +106,17 @@ void ZarrV3Group::ExploreDirectory() const
     {
         if (VSI_ISDIR(psEntry->nMode))
         {
-            const std::string osSubDir = CPLFormFilename(
-                m_osDirectoryName.c_str(), psEntry->pszName, nullptr);
+            std::string osName(psEntry->pszName);
+            while (!osName.empty() &&
+                   (osName.back() == '/' || osName.back() == '\\'))
+                osName.pop_back();
+            if (osName.empty())
+                continue;
+            const std::string osSubDir = CPLFormFilenameSafe(
+                m_osDirectoryName.c_str(), osName.c_str(), nullptr);
             VSIStatBufL sStat;
-            std::string osZarrJsonFilename =
-                CPLFormFilename(osSubDir.c_str(), "zarr.json", nullptr);
+            const std::string osZarrJsonFilename =
+                CPLFormFilenameSafe(osSubDir.c_str(), "zarr.json", nullptr);
             if (VSIStatL(osZarrJsonFilename.c_str(), &sStat) == 0)
             {
                 CPLJSONDocument oDoc;
@@ -126,18 +132,18 @@ void ZarrV3Group::ExploreDirectory() const
                     const std::string osNodeType = oRoot.GetString("node_type");
                     if (osNodeType == "array")
                     {
-                        if (std::find(m_aosArrays.begin(), m_aosArrays.end(),
-                                      psEntry->pszName) == m_aosArrays.end())
+                        if (!cpl::contains(m_oSetArrayNames, osName))
                         {
-                            m_aosArrays.emplace_back(psEntry->pszName);
+                            m_oSetArrayNames.insert(osName);
+                            m_aosArrays.emplace_back(std::move(osName));
                         }
                     }
                     else if (osNodeType == "group")
                     {
-                        if (std::find(m_aosGroups.begin(), m_aosGroups.end(),
-                                      psEntry->pszName) == m_aosGroups.end())
+                        if (!cpl::contains(m_oSetGroupNames, osName))
                         {
-                            m_aosGroups.emplace_back(psEntry->pszName);
+                            m_oSetGroupNames.insert(osName);
+                            m_aosGroups.emplace_back(std::move(osName));
                         }
                     }
                     else
@@ -150,11 +156,11 @@ void ZarrV3Group::ExploreDirectory() const
             }
             else
             {
-                // Implicit group
-                if (std::find(m_aosGroups.begin(), m_aosGroups.end(),
-                              psEntry->pszName) == m_aosGroups.end())
+                // Implicit group (deprecated)
+                if (!cpl::contains(m_oSetGroupNames, osName))
                 {
-                    m_aosGroups.emplace_back(psEntry->pszName);
+                    m_oSetGroupNames.insert(osName);
+                    m_aosGroups.emplace_back(std::move(osName));
                 }
             }
         }
@@ -188,8 +194,8 @@ ZarrV3Group::~ZarrV3Group()
         oRoot.Add("zarr_format", 3);
         oRoot.Add("node_type", "group");
         oRoot.Add("attributes", m_oAttrGroup.Serialize());
-        const std::string osZarrJsonFilename =
-            CPLFormFilename(m_osDirectoryName.c_str(), "zarr.json", nullptr);
+        const std::string osZarrJsonFilename = CPLFormFilenameSafe(
+            m_osDirectoryName.c_str(), "zarr.json", nullptr);
         oDoc.Save(osZarrJsonFilename);
     }
 }
@@ -209,9 +215,9 @@ ZarrV3Group::OpenZarrGroup(const std::string &osName, CSLConstList) const
         return oIter->second;
 
     const std::string osSubDir =
-        CPLFormFilename(m_osDirectoryName.c_str(), osName.c_str(), nullptr);
+        CPLFormFilenameSafe(m_osDirectoryName.c_str(), osName.c_str(), nullptr);
     const std::string osSubDirZarrJsonFilename =
-        CPLFormFilename(osSubDir.c_str(), "zarr.json", nullptr);
+        CPLFormFilenameSafe(osSubDir.c_str(), "zarr.json", nullptr);
 
     VSIStatBufL sStat;
     // Explicit group
@@ -248,6 +254,11 @@ ZarrV3Group::OpenZarrGroup(const std::string &osName, CSLConstList) const
     // Implicit group
     if (VSIStatL(osSubDir.c_str(), &sStat) == 0 && VSI_ISDIR(sStat.st_mode))
     {
+        // Note: Python zarr v3.0.2 still generates implicit groups
+        // See https://github.com/zarr-developers/zarr-python/issues/2794
+        CPLError(CE_Warning, CPLE_AppDefined,
+                 "Support for Zarr V3 implicit group is now deprecated, and "
+                 "may be removed in a future version");
         auto poSubGroup = ZarrV3Group::Create(m_poSharedResource, GetFullName(),
                                               osName, osSubDir);
         poSubGroup->m_poParent =
@@ -286,7 +297,7 @@ std::shared_ptr<ZarrV3Group> ZarrV3Group::CreateOnDisk(
     }
 
     const std::string osZarrJsonFilename(
-        CPLFormFilename(osDirectoryName.c_str(), "zarr.json", nullptr));
+        CPLFormFilenameSafe(osDirectoryName.c_str(), "zarr.json", nullptr));
     VSILFILE *fp = VSIFOpenL(osZarrJsonFilename.c_str(), "wb");
     if (!fp)
     {
@@ -333,8 +344,7 @@ ZarrV3Group::CreateGroup(const std::string &osName,
 
     GetGroupNames();
 
-    if (std::find(m_aosGroups.begin(), m_aosGroups.end(), osName) !=
-        m_aosGroups.end())
+    if (cpl::contains(m_oSetGroupNames, osName))
     {
         CPLError(CE_Failure, CPLE_AppDefined,
                  "A group with same name already exists");
@@ -342,7 +352,7 @@ ZarrV3Group::CreateGroup(const std::string &osName,
     }
 
     const std::string osDirectoryName =
-        CPLFormFilename(m_osDirectoryName.c_str(), osName.c_str(), nullptr);
+        CPLFormFilenameSafe(m_osDirectoryName.c_str(), osName.c_str(), nullptr);
     auto poGroup = CreateOnDisk(m_poSharedResource, GetFullName(), osName,
                                 osDirectoryName);
     if (!poGroup)
@@ -417,6 +427,12 @@ static CPLJSONObject FillDTypeElts(const GDALExtendedDataType &oDataType,
             dtype.Set(dummy, "int64");
             break;
         }
+        case GDT_Float16:
+        {
+            elt.nativeType = DtypeElt::NativeType::IEEEFP;
+            dtype.Set(dummy, "float16");
+            break;
+        }
         case GDT_Float32:
         {
             elt.nativeType = DtypeElt::NativeType::IEEEFP;
@@ -436,6 +452,12 @@ static CPLJSONObject FillDTypeElts(const GDALExtendedDataType &oDataType,
             bUnsupported = true;
             break;
         }
+        case GDT_CFloat16:
+        {
+            elt.nativeType = DtypeElt::NativeType::COMPLEX_IEEEFP;
+            dtype.Set(dummy, "complex32");
+            break;
+        }
         case GDT_CFloat32:
         {
             elt.nativeType = DtypeElt::NativeType::COMPLEX_IEEEFP;
@@ -450,8 +472,8 @@ static CPLJSONObject FillDTypeElts(const GDALExtendedDataType &oDataType,
         }
         case GDT_TypeCount:
         {
-            static_assert(GDT_TypeCount == GDT_Int8 + 1,
-                          "GDT_TypeCount == GDT_Int8 + 1");
+            static_assert(GDT_TypeCount == GDT_CFloat16 + 1,
+                          "GDT_TypeCount == GDT_CFloat16 + 1");
             break;
         }
     }
@@ -520,8 +542,7 @@ std::shared_ptr<GDALMDArray> ZarrV3Group::CreateMDArray(
 
     GetMDArrayNames();
 
-    if (std::find(m_aosArrays.begin(), m_aosArrays.end(), osName) !=
-        m_aosArrays.end())
+    if (cpl::contains(m_oSetArrayNames, osName))
     {
         CPLError(CE_Failure, CPLE_AppDefined,
                  "An array with same name already exists");
@@ -537,7 +558,7 @@ std::shared_ptr<GDALMDArray> ZarrV3Group::CreateMDArray(
         CSLFetchNameValueDef(papszOptions, "DIM_SEPARATOR", "/");
 
     const std::string osArrayDirectory =
-        CPLFormFilename(m_osDirectoryName.c_str(), osName.c_str(), nullptr);
+        CPLFormFilenameSafe(m_osDirectoryName.c_str(), osName.c_str(), nullptr);
     if (VSIMkdir(osArrayDirectory.c_str(), 0755) != 0)
     {
         VSIStatBufL sStat;
@@ -568,13 +589,13 @@ std::shared_ptr<GDALMDArray> ZarrV3Group::CreateMDArray(
         oCodecs.Add(oCodec);
     }
 
-    // Not documented
-    const char *pszEndian = CSLFetchNameValue(papszOptions, "@ENDIAN");
-    if (pszEndian)
+    // Not documented option, but 'bytes' codec is required
+    const char *pszEndian =
+        CSLFetchNameValueDef(papszOptions, "@ENDIAN", "little");
     {
         CPLJSONObject oCodec;
-        oCodec.Add("name", "endian");
-        oCodec.Add("configuration", ZarrV3CodecEndian::GetConfiguration(
+        oCodec.Add("name", "bytes");
+        oCodec.Add("configuration", ZarrV3CodecBytes::GetConfiguration(
                                         EQUAL(pszEndian, "little")));
         oCodecs.Add(oCodec);
     }
@@ -643,6 +664,18 @@ std::shared_ptr<GDALMDArray> ZarrV3Group::CreateMDArray(
                                                       typesize, blocksize));
         oCodecs.Add(oCodec);
     }
+    else if (EQUAL(pszCompressor, "ZSTD"))
+    {
+        CPLJSONObject oCodec;
+        oCodec.Add("name", "zstd");
+        const char *pszLevel =
+            CSLFetchNameValueDef(papszOptions, "ZSTD_LEVEL", "13");
+        const bool bChecksum = CPLTestBool(
+            CSLFetchNameValueDef(papszOptions, "ZSTD_CHECKSUM", "FALSE"));
+        oCodec.Add("configuration", ZarrV3CodecZstd::GetConfiguration(
+                                        atoi(pszLevel), bChecksum));
+        oCodecs.Add(oCodec);
+    }
     else if (!EQUAL(pszCompressor, "NONE"))
     {
         CPLError(CE_Failure, CPLE_AppDefined,
@@ -672,11 +705,17 @@ std::shared_ptr<GDALMDArray> ZarrV3Group::CreateMDArray(
     if (!poArray)
         return nullptr;
     poArray->SetNew(true);
-    std::string osFilename =
-        CPLFormFilename(osArrayDirectory.c_str(), "zarr.json", nullptr);
+    const std::string osFilename =
+        CPLFormFilenameSafe(osArrayDirectory.c_str(), "zarr.json", nullptr);
     poArray->SetFilename(osFilename);
     poArray->SetDimSeparator(pszDimSeparator);
     poArray->SetDtype(dtype);
+    if (oCodecs.Size() > 0 &&
+        oCodecs[oCodecs.Size() - 1].GetString("name") != "bytes")
+    {
+        poArray->SetStructuralInfo(
+            "COMPRESSOR", oCodecs[oCodecs.Size() - 1].ToString().c_str());
+    }
     if (poCodecs)
         poArray->SetCodecs(std::move(poCodecs));
     poArray->SetUpdatable(true);
