@@ -171,14 +171,15 @@ class OGRFeatherWriterLayer final : public OGRArrowWriterLayer
         return true;
     }
 
+    friend class OGRFeatherWriterDataset;
+    bool Close();
+
   public:
     OGRFeatherWriterLayer(
         GDALDataset *poDS, arrow::MemoryPool *poMemoryPool,
         const std::shared_ptr<arrow::io::OutputStream> &poOutputStream,
         const char *pszLayerName);
 
-    ~OGRFeatherWriterLayer() override;
-
     bool SetOptions(const std::string &osFilename, CSLConstList papszOptions,
                     const OGRSpatialReference *poSpatialRef,
                     OGRwkbGeometryType eGType);
@@ -209,6 +210,10 @@ class OGRFeatherWriterDataset final : public GDALPamDataset
         const char *pszFilename,
         const std::shared_ptr<arrow::io::OutputStream> &poOutputStream);
 
+    ~OGRFeatherWriterDataset() override;
+
+    CPLErr Close() override;
+
     arrow::MemoryPool *GetMemoryPool() const
     {
         return m_poMemoryPool.get();
