@@ -172,7 +172,7 @@ int OGRDXFDataSource::Open(const char *pszFilename, bool bHeaderOnly,
     /* -------------------------------------------------------------------- */
     else /* if( EQUAL(szLineBuf,"HEADER") ) */
     {
-        if (!ReadHeaderSection())
+        if (!ReadHeaderSection(papszOptionsIn))
             return FALSE;
         if (ReadValue(szLineBuf) < 0)
         {
@@ -802,7 +802,7 @@ bool OGRDXFDataSource::LookupDimStyle(
 /*                         ReadHeaderSection()                          */
 /************************************************************************/
 
-bool OGRDXFDataSource::ReadHeaderSection()
+bool OGRDXFDataSource::ReadHeaderSection(CSLConstList papszOpenOptionsIn)
 
 {
     char szLineBuf[257];
@@ -823,6 +823,8 @@ bool OGRDXFDataSource::ReadHeaderSection()
         }
 
         oHeaderVariables[l_osName] = szLineBuf;
+        GDALDataset::SetMetadataItem(l_osName.c_str(), szLineBuf,
+                                     "DXF_HEADER_VARIABLES");
     }
     if (nCode < 0)
     {
@@ -859,6 +861,8 @@ bool OGRDXFDataSource::ReadHeaderSection()
             }
 
             oHeaderVariables[l_osName] = szLineBuf;
+            GDALDataset::SetMetadataItem(l_osName.c_str(), szLineBuf,
+                                         "DXF_HEADER_VARIABLES");
         }
         if (nCode < 0)
         {
@@ -889,7 +893,9 @@ bool OGRDXFDataSource::ReadHeaderSection()
         osEncoding = CPL_ENC_ISO8859_1;
     }
 
-    const char *pszEncoding = CPLGetConfigOption("DXF_ENCODING", nullptr);
+    const char *pszEncoding =
+        CSLFetchNameValueDef(papszOpenOptionsIn, "ENCODING",
+                             CPLGetConfigOption("DXF_ENCODING", nullptr));
     if (pszEncoding != nullptr)
         osEncoding = pszEncoding;
 
