@@ -1,5 +1,4 @@
 /******************************************************************************
- * $Id$
  *
  * Project:  Elasticsearch Translator
  * Purpose:
@@ -177,7 +176,7 @@ class OGRElasticLayer final : public OGRLayer
     virtual OGRErr CreateGeomField(const OGRGeomFieldDefn *poField,
                                    int bApproxOK) override;
 
-    virtual const char *GetName() override
+    const char *GetName() override
     {
         return m_poFeatureDefn->GetName();
     }
@@ -189,21 +188,13 @@ class OGRElasticLayer final : public OGRLayer
 
     virtual GIntBig GetFeatureCount(int bForce) override;
 
-    virtual void SetSpatialFilter(OGRGeometry *poGeom) override
-    {
-        SetSpatialFilter(0, poGeom);
-    }
+    OGRErr ISetSpatialFilter(int iGeomField,
+                             const OGRGeometry *poGeom) override;
 
-    virtual void SetSpatialFilter(int iGeomField, OGRGeometry *poGeom) override;
     virtual OGRErr SetAttributeFilter(const char *pszFilter) override;
 
-    virtual OGRErr GetExtent(OGREnvelope *psExtent, int bForce = TRUE) override
-    {
-        return GetExtent(0, psExtent, bForce);
-    }
-
-    virtual OGRErr GetExtent(int iGeomField, OGREnvelope *psExtent,
-                             int bForce = TRUE) override;
+    virtual OGRErr IGetExtent(int iGeomField, OGREnvelope *psExtent,
+                              bool bForce) override;
 
     virtual OGRErr SyncToDisk() override;
 
@@ -307,8 +298,8 @@ class OGRElasticAggregationLayer final
     GIntBig GetFeatureCount(int bForce) override;
     int TestCapability(const char *) override;
 
-    using OGRLayer::SetSpatialFilter;
-    void SetSpatialFilter(OGRGeometry *poGeom) override;
+    OGRErr ISetSpatialFilter(int iGeomField,
+                             const OGRGeometry *poGeom) override;
 
     GDALDataset *GetDataset() override;
 
@@ -367,7 +358,7 @@ class OGRElasticDataSource final : public GDALDataset
         return m_osURL.c_str();
     }
 
-    virtual const char *GetName()
+    const char *GetName()
     {
         return m_pszName;
     }
