@@ -1,5 +1,4 @@
 /******************************************************************************
- * $Id$
  *
  * Project:  OpenGIS Simple Features Reference Implementation
  * Purpose:  Defines OGRWarpedLayer class
@@ -17,13 +16,22 @@
 #ifndef DOXYGEN_SKIP
 
 #include "ogrlayerdecorator.h"
+#include "ogrlayerwithtranslatefeature.h"
+
 #include <memory>
 
+#if defined(_MSC_VER)
+#pragma warning(push)
+// Silence warnings of the type warning C4250: 'OGRWarpedLayer': inherits 'OGRLayerDecorator::OGRLayerDecorator::GetMetadata' via dominance
+#pragma warning(disable : 4250)
+#endif
+
 /************************************************************************/
 /*                           OGRWarpedLayer                             */
 /************************************************************************/
 
-class CPL_DLL OGRWarpedLayer : public OGRLayerDecorator
+class CPL_DLL OGRWarpedLayer : public OGRLayerDecorator,
+                               public OGRLayerWithTranslateFeature
 {
     CPL_DISALLOW_COPY_ASSIGN(OGRWarpedLayer)
 
@@ -31,9 +39,11 @@ class CPL_DLL OGRWarpedLayer : public OGRLayerDecorator
     OGRFeatureDefn *m_poFeatureDefn;
     int m_iGeomField;
 
-    OGRCoordinateTransformation *m_poCT;
-    OGRCoordinateTransformation *m_poReversedCT; /* may be NULL */
-    OGRSpatialReference *m_poSRS;
+    OGRGeometryFactory::TransformWithOptionsCache m_transformCacheForward{};
+    OGRGeometryFactory::TransformWithOptionsCache m_transformCacheReverse{};
+    OGRCoordinateTransformation *m_poCT = nullptr;
+    OGRCoordinateTransformation *m_poReversedCT = nullptr; /* may be NULL */
+    OGRSpatialReference *m_poSRS = nullptr;
 
     OGREnvelope sStaticEnvelope{};
 
@@ -54,15 +64,14 @@ class CPL_DLL OGRWarpedLayer : public OGRLayerDecorator
             poReversedCT /* may be NULL, ownership acquired by OGRWarpedLayer */);
     virtual ~OGRWarpedLayer();
 
+    void TranslateFeature(
+        std::unique_ptr<OGRFeature> poSrcFeature,
+        std::vector<std::unique_ptr<OGRFeature>> &apoOutFeatures) override;
+
     void SetExtent(double dfXMin, double dfYMin, double dfXMax, double dfYMax);
 
-    virtual void SetSpatialFilter(OGRGeometry *) override;
-    virtual void SetSpatialFilterRect(double dfMinX, double dfMinY,
-                                      double dfMaxX, double dfMaxY) override;
-    virtual void SetSpatialFilter(int iGeomField, OGRGeometry *) override;
-    virtual void SetSpatialFilterRect(int iGeomField, double dfMinX,
-                                      double dfMinY, double dfMaxX,
-                                      double dfMaxY) override;
+    virtual OGRErr ISetSpatialFilter(int iGeomField,
+                                     const OGRGeometry *) override;
 
     virtual OGRFeature *GetNextFeature() override;
     virtual OGRFeature *GetFeature(GIntBig nFID) override;
@@ -80,9 +89,8 @@ class CPL_DLL OGRWarpedLayer : public OGRLayerDecorator
     virtual OGRSpatialReference *GetSpatialRef() override;
 
     virtual GIntBig GetFeatureCount(int bForce = TRUE) override;
-    virtual OGRErr GetExtent(int iGeomField, OGREnvelope *psExtent,
-                             int bForce = TRUE) override;
-    virtual OGRErr GetExtent(OGREnvelope *psExtent, int bForce = TRUE) override;
+    virtual OGRErr IGetExtent(int iGeomField, OGREnvelope *psExtent,
+                              bool bForce = true) override;
 
     virtual int TestCapability(const char *) override;
 
@@ -90,6 +98,10 @@ class CPL_DLL OGRWarpedLayer : public OGRLayerDecorator
                                 CSLConstList papszOptions = nullptr) override;
 };
 
+#if defined(_MSC_VER)
+#pragma warning(pop)
+#endif
+
 #endif /* #ifndef DOXYGEN_SKIP */
 
 #endif  //  OGRWARPEDLAYER_H_INCLUDED
