@@ -1107,11 +1107,19 @@ int GMLHandler::FindRealPropertyByCheckingConditions(int nIdx, void *attr)
 OGRErr GMLHandler::startElementFeatureAttribute(const char *pszName,
                                                 int nLenName, void *attr)
 {
-    /* Reset flag */
-    m_bInCurField = false;
 
     GMLReadState *poState = m_poReader->GetState();
 
+    if (m_bInCurField && m_nAttributeIndex >= 0 &&
+        std::string_view(pszName, nLenName) == "timePosition")
+    {
+        poState->PushPath(pszName, nLenName);
+        return OGRERR_NONE;
+    }
+
+    /* Reset flag */
+    m_bInCurField = false;
+
     /* -------------------------------------------------------------------- */
     /*      If we are collecting geometry, or if we determine this is a     */
     /*      geometry element then append to the geometry info.              */
@@ -1182,6 +1190,14 @@ OGRErr GMLHandler::startElementFeatureAttribute(const char *pszName,
                        "RouteSegment") == 0)
                 bReadGeometry = strcmp(pszName, "Curve") == 0;
 
+            // AIXM special case: we want to read both horizontalProjection_location and
+            // horizontalProjection_linearExtent
+            else if (eAppSchemaType == APPSCHEMA_AIXM &&
+                     STARTS_WITH(poState->m_poFeature->GetClass()->GetName(),
+                                 "horizontalProjection_") == 0)
+                bReadGeometry =
+                    STARTS_WITH(pszName, "horizontalProjection_") == 0;
+
             /* For Inspire objects : the "main" geometry is in a <geometry>
              * element */
             else if (m_bAlreadyFoundGeometry)
@@ -1546,13 +1562,12 @@ OGRErr GMLHandler::endElementBoundedByInFeature()
 }
 
 /************************************************************************/
-/*                       ParseAIXMElevationPoint()                      */
+/*                      ParseAIXMElevationProperties()                  */
 /************************************************************************/
 
-CPLXMLNode *GMLHandler::ParseAIXMElevationPoint(CPLXMLNode *psGML)
+void GMLHandler::ParseAIXMElevationProperties(const CPLXMLNode *psGML)
 {
-    const char *pszElevation = CPLGetXMLValue(psGML, "elevation", nullptr);
-    if (pszElevation)
+    if (const char *pszElevation = CPLGetXMLValue(psGML, "elevation", nullptr))
     {
         m_poReader->SetFeaturePropertyDirectly("elevation",
                                                CPLStrdup(pszElevation), -1);
@@ -1565,9 +1580,8 @@ CPLXMLNode *GMLHandler::ParseAIXMElevationPoint(CPLXMLNode *psGML)
         }
     }
 
-    const char *pszGeoidUndulation =
-        CPLGetXMLValue(psGML, "geoidUndulation", nullptr);
-    if (pszGeoidUndulation)
+    if (const char *pszGeoidUndulation =
+            CPLGetXMLValue(psGML, "geoidUndulation", nullptr))
     {
         m_poReader->SetFeaturePropertyDirectly(
             "geoidUndulation", CPLStrdup(pszGeoidUndulation), -1);
@@ -1580,6 +1594,36 @@ CPLXMLNode *GMLHandler::ParseAIXMElevationPoint(CPLXMLNode *psGML)
         }
     }
 
+    if (const char *pszVerticalDatum =
+            CPLGetXMLValue(psGML, "verticalDatum", nullptr))
+    {
+        m_poReader->SetFeaturePropertyDirectly("verticalDatum",
+                                               CPLStrdup(pszVerticalDatum), -1);
+    }
+
+    if (const char *pszVerticalAccuracy =
+            CPLGetXMLValue(psGML, "verticalAccuracy", nullptr))
+    {
+        m_poReader->SetFeaturePropertyDirectly(
+            "verticalAccuracy", CPLStrdup(pszVerticalAccuracy), -1);
+        const char *pszVerticalAccuracyUnit =
+            CPLGetXMLValue(psGML, "verticalAccuracy.uom", nullptr);
+        if (pszVerticalAccuracyUnit)
+        {
+            m_poReader->SetFeaturePropertyDirectly(
+                "verticalAccuracy_uom", CPLStrdup(pszVerticalAccuracyUnit), -1);
+        }
+    }
+}
+
+/************************************************************************/
+/*                       ParseAIXMElevationPoint()                      */
+/************************************************************************/
+
+CPLXMLNode *GMLHandler::ParseAIXMElevationPoint(CPLXMLNode *psGML)
+{
+    ParseAIXMElevationProperties(psGML);
+
     const char *pszPos = CPLGetXMLValue(psGML, "pos", nullptr);
     const char *pszCoordinates = CPLGetXMLValue(psGML, "coordinates", nullptr);
     if (pszPos != nullptr || pszCoordinates != nullptr)
@@ -1691,6 +1735,13 @@ OGRErr GMLHandler::endElementGeometry()
         {
             psInterestNode = ParseAIXMElevationPoint(psInterestNode);
         }
+        else if (eAppSchemaType == APPSCHEMA_AIXM &&
+                 psInterestNode != nullptr &&
+                 (strcmp(psInterestNode->pszValue, "ElevatedCurve") == 0 ||
+                  strcmp(psInterestNode->pszValue, "ElevateSurface") == 0))
+        {
+            ParseAIXMElevationProperties(psInterestNode);
+        }
         else if (eAppSchemaType == APPSCHEMA_MTKGML &&
                  psInterestNode != nullptr)
         {
@@ -2085,6 +2136,7 @@ bool GMLHandler::IsGeometryElement(const char *pszElement)
 
     if (eAppSchemaType == APPSCHEMA_AIXM &&
         (strcmp(pszElement, "ElevatedPoint") == 0 ||
+         strcmp(pszElement, "ElevatedCurve") == 0 ||
          strcmp(pszElement, "ElevatedSurface") == 0))
     {
         return true;
